import numpy as np
from matplotlib import pyplot as plt

# Define function
f_opt = lambda t: np.cos(2*np.pi*t)


# Number of samples
M = 100

# Degree + 1
N = 9

# Choose sample location
tt = np.linspace(-1, 1, M)

# Compute "clean" function
ff = f_opt(tt)

# Form "observation matrix"
A = np.zeros((M,N))
for i in np.arange(N):
    A[:,i] = tt ** i
    

# Compute and plot least squares polynomial fit
xhat = np.linalg.pinv(A) @ ff
fhat = A @ xhat

plt.clf()
plt.plot(tt, ff, 'b')
plt.plot(tt, fhat, 'r--' )
plt.xlabel('t')
plt.title('Polynomial interpolation of f(t), Degree = {}'.format(N-1))
plt.legend(('Original f(t)','Polynomial fit'))


## Construct noisy observations
np.random.seed(2020) # Set random seed so results are repeatable

# Random gaussian noise
noise1 = np.random.randn(M)
noise1 = 5*noise1/np.linalg.norm(noise1)
y1 = ff + noise1

# Sparse gaussian noise
K = 15
Gamma = np.random.choice(M,K,0) 
noise2 = np.zeros(M)
noise2[Gamma] = np.random.randn(K)
noise2 = 5*noise2/np.linalg.norm(noise2)
y2 = ff + noise2


# Uniform noise
noise3 = np.random.rand(M)-0.5
noise3 = 5*noise3/np.linalg.norm(noise3)
y3 = ff + noise3
